#include "architecture.h"
#include "builder.h"
#include "clangcompiler.h"
#include "gnuarchiver.h"
#include "gnucompiler.h"
#include "gnulinker.h"
#include "mingwdlltool.h"
#include "systemtools.h"

using namespace std;

SystemTools::SystemTools(Builder &builder, const Architecture &arch)
{
	const string &sys = arch.get_system();
	if((sys=="darwin" || sys=="freebsd") && builder.get_vfs().find_binary("clang"))
	{
		add_tool(new ClangCompiler(builder, arch, "CC"));
		add_tool(new ClangCompiler(builder, arch, "CXX"));
		add_tool(new ClangCompiler(builder, arch, "OBJC"));
	}
	else
	{
		add_tool(new GnuCompiler(builder, arch, "CC"));
		add_tool(new GnuCompiler(builder, arch, "CXX"));
		add_tool(new GnuCompiler(builder, arch, "OBJC"));
	}

	add_tool(new GnuLinker(builder, arch));
	add_tool(new GnuArchiver(builder, arch));
	if(arch.get_system()=="windows")
		add_tool(new MingwDllTool(builder, arch));
}
